#colour_map
# Python script which loads PNG image files, extracts R,G,B colour matrices
# and re-colours according to an interpolation of a defined colour map.
#
# http://www.tayloredmktg.com/rgb/
#
# LAST UPDATED by Andy French May 2018.

#Import required libraries
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.image as mpimg
import numpy as np

#

#Define image filename
image_filename = "sybil.png"

#Define colourmap ("B&W", "duotone","four colour", "jet", "prism")
colour_map_type = "B&W"

#

#MATLAB style colour map generating functions
def matlab_colourmaps( colour_map_type, k ):

    if colour_map_type=="four colour":

        cmap_levels = [0,0.2,0.5,0.8]
        cmap = [
            [1,0,0],
            [0,1,0],
            [0,0,1],
            [0,0,0]
               ]

    elif colour_map_type=="duotone":

        cmap = [
            [1,0,0],[0,1,0]
               ]
        cmap_levels = [0,1]

    elif colour_map_type=="B&W":

        cmap = [[0,0,0]]
        cmap_levels = np.linspace(0,1,k)

        for i in range(1,k):
            if (i%2)==0:
                cmap.append( [0,0,0] )
            else:
                cmap.append( [1,1,1] )

    elif colour_map_type=="prism":

        cmap = [
            [1, 0, 0],
            [1,0.5,0],
            [1.0000,1.0000,0],
            [0,1.0000,0],
            [0,0,1.0000],
            [0.6667,0,1.0000],
            [1.0000,0,0],
            [1.0000,0.5000,0],
            [1.0000,1.0000,0],
            [0,1.0000,0],
            [0,0,1.0000],
            [0.6667,0,1.0000],
            [1.0000,0,0],
            [1.0000,0.5000,0],
            [1.0000,1.0000,0],
            [0,1.0000,0],
            [0,0,1.0000],
            [0.6667,0,1.0000],
            [1.0000,0,0],
            [1.0000,0.5000,0],
            [1.0000,1.0000,0],
            [0,1.0000,0],
            [0,0,1.0000],
            [0.6667,0,1.0000],
            [1.0000,0,0],
            [1.0000,0.5000,0],
            [1.0000,1.0000,0],
            [0,1.0000,0],
            [0,0,1.0000],
            [0.6667,0,1.0000],
            [1.0000,0,0],
            [1.0000,0.5000,0],
            [1.0000,1.0000,0],
            [0,1.0000,0],
            [0,0,1.0000],
            [0.6667,0,1.0000],
            [1.0000,0,0],
            [1.0000,0.5000,0],
            [1.0000,1.0000,0],
            [0,1.0000,0],
            [0,0,1.0000],
            [0.6667,0,1.0000],
            [1.0000,0,0],
            [1.0000,0.5000,0],
            [1.0000,1.0000,0],
            [0,1.0000,0],
            [0,0,1.0000],
            [0.6667,0,1.0000],
            [1.0000,0,0],
            [1.0000,0.5000,0],
            [1.0000,1.0000,0],
            [0,1.0000,0],
            [0,0,1.0000],
            [0.6667,0,1.0000],
            [1.0000,0,0],
            [1.0000,0.5000,0],
            [1.0000,1.0000,0],
            [0,1.0000,0],
            [0,0,1.0000],
            [0.6667,0,1.0000],
            [1.0000,0,0],
            [1.0000,0.5000,0],
            [1.0000,1.0000,0],
            [0,1.0000,0]
            ]
        
        #Define linearaly spaced vector [0,1] which corresponds to colour map colours
        cmap_levels = np.linspace(0,1,len(cmap))

    else:
        #MATLAB default "jet" colour map
        cmap = [
            [0,0,0.5625],
            [0,0,0.6250],
            [0,0,0.6875],
            [0,0,0.7500],
            [0,0,0.8125],
            [0,0,0.8750],
            [0,0,0.9375],
            [0,0,1.0000],
            [0,0.0625,1.0000],
            [0,0.1250,1.0000],
            [0,0.1875,1.0000],
            [0,0.2500,1.0000],
            [0,0.3125,1.0000],
            [0,0.3750,1.0000],
            [0,0.4375,1.0000],
            [0,0.5000,1.0000],
            [0,0.5625,1.0000],
            [0,0.6250,1.0000],
            [0,0.6875,1.0000],
            [0,0.7500,1.0000],
            [0,0.8125,1.0000],
            [0,0.8750,1.0000],
            [0,0.9375,1.0000],
            [0,1.0000,1.0000],
            [0.0625,1.0000,0.9375],
            [0.1250,1.0000,0.8750],
            [0.1875,1.0000,0.8125],
            [0.2500,1.0000,0.7500],
            [0.3125,1.0000,0.6875],
            [0.3750,1.0000,0.6250],
            [0.4375,1.0000,0.5625],
            [0.5000,1.0000,0.5000],
            [0.5625,1.0000,0.4375],
            [0.6250,1.0000,0.3750],
            [0.6875,1.0000,0.3125],
            [0.7500,1.0000,0.2500],
            [0.8125,1.0000,0.1875],
            [0.8750,1.0000,0.1250],
            [0.9375,1.0000,0.0625],
            [1.0000,1.0000,0],
            [1.0000,0.9375,0],
            [1.0000,0.8750,0],
            [1.0000,0.8125,0],
            [1.0000,0.7500,0],
            [1.0000,0.6875,0],
            [1.0000,0.6250,0],
            [1.0000,0.5625,0],
            [1.0000,0.5000,0],
            [1.0000,0.4375,0],
            [1.0000,0.3750,0],
            [1.0000,0.3125,0],
            [1.0000,0.2500,0],
            [1.0000,0.1875,0],
            [1.0000,0.1250,0],
            [1.0000,0.0625,0],
            [1.0000,0,0],
            [0.9375,0,0],
            [0.8750,0,0],
            [0.8125,0,0],
            [0.7500,0,0],
            [0.6875,0,0],
            [0.6250,0,0],
            [0.5625,0,0],
            [0.5000,0,0]
            ]

        #Define linearaly spaced vector [0,1] which corresponds to colour map colours
        cmap_levels = np.linspace(0,1,len(cmap))

    #Define vectors for red, green, blue colour map values
    z = np.zeros((1,len(cmap)))[0]
    cmap_red = z.copy()
    cmap_green = z.copy()
    cmap_blue = z.copy()
    for i in range(0,len(cmap)):
        cmap_red[i] = cmap[i][0]
        cmap_green[i] = cmap[i][1]
        cmap_blue[i] = cmap[i][2]
           
    return cmap, cmap_levels, cmap_red, cmap_green, cmap_blue

#


#Image import and RGB segmentation function. Call as:
# rgb, R, G, B, red, green, blue, rows, cols, isgray = rgb_import( image_filename )
def rgb_import( image_filename ):

    #Import PNG image and extract R,G,B colour planes. These will be
    #stored as numpy arrays of dimensions rows x cols, with floating point
    #numbert elements betwen 0 and 1
    print(" \n Importing PNG image " + image_filename + " ... ")
    rgb= mpimg.imread( image_filename )
    dims = np.shape(rgb)
    rows = dims[0]
    cols = dims[1]
    z = np.zeros((rows,cols))
    R = z.copy()
    G = z.copy()
    B = z.copy()
    if len(dims)==2:
        #Grayscale
        R[:,:] = (1/3)*rgb[:,:]
        G[:,:] = (1/3)*rgb[:,:]
        B[:,:] = (1/3)*rgb[:,:]
        isgray = 1
    else:
        #Full colour
        R[:,:] = rgb[:,:,0]
        G[:,:] = rgb[:,:,1]
        B[:,:] = rgb[:,:,2]
        isgray = 0

    #Construct red,green,blue channels
    z = np.zeros((rows,cols,3))
    red = z.copy()
    green = z.copy()
    blue = z.copy()
    red[:,:,0] = R[:,:]
    green[:,:,1] = G[:,:]
    blue[:,:,2] = B[:,:]

    #Set return values of function
    return rgb, R, G, B, red, green, blue, rows, cols, isgray

#

#Import image and perform colour separations
rgb, R, G, B, red, green, blue, rows, cols, isgray = rgb_import( image_filename )

#Determine colour function values for each pixel
C = (1/3) * ( R + G + B )

#Get colourmap
cmap, cmap_levels, cmap_red, cmap_green, cmap_blue = matlab_colourmaps( colour_map_type, 20 )

#Form updated rgb colour, interpolating colour map using colour function C
print(" \n Interpolating colour map for " + image_filename + " ... ")
new_rgb = rgb.copy()
new_rgb[:,:,0] = np.interp( C, cmap_levels, cmap_red )
new_rgb[:,:,1] = np.interp( C, cmap_levels , cmap_green )
new_rgb[:,:,2] = np.interp( C, cmap_levels , cmap_blue )

#Create figure and axis
print(" \n Plotting RGB image with threshold colour ... ")
fig = plt.figure(figsize=(10, 8))

#Plot image with axis and save as PNG bitmap
imgplot = plt.imshow(new_rgb)
plt.title( image_filename + " using new colourmap " )
fig.savefig( image_filename.replace(".png","_colormap_axis.png"), dpi=300)
plt.close()

plt.imsave( image_filename.replace(".png","_colormap.png"), new_rgb )

#Wait for user input to close terminal window
input("\n Press return to close ")

#End of code




