#colour_separations
# Python script which loads PNG image files, extracts R,G,B colour matrices
# and then saves PNG images of the associated colour separations.
#
# LAST UPDATED by Andy French January 2018.

#Define image filename
image_filename = "sybil.png"
image_filename = "kipling.png"

#Import required libraries
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.image as mpimg
import numpy as np

#

#Image import and RGB segmentation function. Call as:
# rgb, R, G, B, red, green, blue, rows, cols, isgray = rgb_import( image_filename )
def rgb_import( image_filename ):

    #Import PNG image and extract R,G,B colour planes. These will be
    #stored as numpy arrays of dimensions rows x cols, with floating point
    #numbert elements betwen 0 and 1
    print(" \n Importing PNG image " + image_filename + " ... ")
    rgb= mpimg.imread( image_filename )
    dims = np.shape(rgb)
    rows = dims[0]
    cols = dims[1]
    z = np.zeros((rows,cols))
    R = z.copy()
    G = z.copy()
    B = z.copy()
    if len(dims)==2:
        #Grayscale
        R[:,:] = (1/3)*rgb[:,:]
        G[:,:] = (1/3)*rgb[:,:]
        B[:,:] = (1/3)*rgb[:,:]
        isgray = 1
    else:
        #Full colour
        R[:,:] = rgb[:,:,0]
        G[:,:] = rgb[:,:,1]
        B[:,:] = rgb[:,:,2]
        isgray = 0

    #Construct red,green,blue channels
    z = np.zeros((rows,cols,3))
    red = z.copy()
    green = z.copy()
    blue = z.copy()
    red[:,:,0] = R[:,:]
    green[:,:,1] = G[:,:]
    blue[:,:,2] = B[:,:]

    #Set return values of function
    return rgb, R, G, B, red, green, blue, rows, cols, isgray

#

#Import image and perform colour separations
rgb, R, G, B, red, green, blue, rows, cols, isgray = rgb_import( image_filename )

#Create figure and axis
print(" \n Plotting image colour separations ... ")
fig = plt.figure(figsize=(10, 8))

#Plot image with axis
imgplot = plt.imshow(rgb)
plt.title(image_filename)
fig.savefig( image_filename.replace(".png","_with axis.png"), dpi=300)

#Plot R,G,B components
plt.imsave( image_filename.replace(".png","_red.png"), red )
plt.imsave( image_filename.replace(".png","_green.png"), green )
plt.imsave( image_filename.replace(".png","_blue.png"), blue )

#Wait for user input to close terminal window
input("\n Press return to close ")

#End of code




