#colour_threshold
# Python script which loads PNG image files, extracts R,G,B colour matrices
# and re-colours regions with a define colour according to a theshold criteria.
# For example, if R,G,B values are over 0.7 (values range from 0 to 1), colour these
# pixels with a deep pink colour [1,20/255,147/255]
#
# http://www.tayloredmktg.com/rgb/
#
# LAST UPDATED by Andy French May 2018.

#Define image filename
image_filename = "sybil.png"

#Set threshold colour and level
thresh = 0.7
thresh_rgb = [1,20/255,147/255]

#

#Import required libraries
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.image as mpimg
import numpy as np

#

#Image import and RGB segmentation function. Call as:
# rgb, R, G, B, red, green, blue, rows, cols, isgray = rgb_import( image_filename )
def rgb_import( image_filename ):

    #Import PNG image and extract R,G,B colour planes. These will be
    #stored as numpy arrays of dimensions rows x cols, with floating point
    #numbert elements betwen 0 and 1
    print(" \n Importing PNG image " + image_filename + " ... ")
    rgb= mpimg.imread( image_filename )
    dims = np.shape(rgb)
    rows = dims[0]
    cols = dims[1]
    z = np.zeros((rows,cols))
    R = z.copy()
    G = z.copy()
    B = z.copy()
    if len(dims)==2:
        #Grayscale
        R[:,:] = (1/3)*rgb[:,:]
        G[:,:] = (1/3)*rgb[:,:]
        B[:,:] = (1/3)*rgb[:,:]
        isgray = 1
    else:
        #Full colour
        R[:,:] = rgb[:,:,0]
        G[:,:] = rgb[:,:,1]
        B[:,:] = rgb[:,:,2]
        isgray = 0

    #Construct red,green,blue channels
    z = np.zeros((rows,cols,3))
    red = z.copy()
    green = z.copy()
    blue = z.copy()
    red[:,:,0] = R[:,:]
    green[:,:,1] = G[:,:]
    blue[:,:,2] = B[:,:]

    #Set return values of function
    return rgb, R, G, B, red, green, blue, rows, cols, isgray

#

#Import image and perform colour separations
rgb, R, G, B, red, green, blue, rows, cols, isgray = rgb_import( image_filename )

#Apply threshold logic to copies of R,G,B colour matrices
#Change + to * for AND logic. Default is OR logic
RR = R.copy()
GG = G.copy()
BB = B.copy()
RR[ (R > thresh)+(G > thresh)+(B > thresh) ] = thresh_rgb[0]
GG[ (R > thresh)+(G > thresh)+(B > thresh) ] = thresh_rgb[1]
BB[ (R > thresh)+(G > thresh)+(B > thresh) ] = thresh_rgb[2]

#Form updated rgb colour array for plotting
new_rgb = rgb.copy()
new_rgb[:,:,0] = RR[:,:]
new_rgb[:,:,1] = GG[:,:]
new_rgb[:,:,2] = BB[:,:]

#Create figure and axis
print(" \n Plotting RGB image with threshold colour ... ")
fig = plt.figure(figsize=(10, 8))

#Plot image with axis and save as PNG bitmap
imgplot = plt.imshow(new_rgb)
plt.title( image_filename + ". RGB threshold = " + str(thresh) )
fig.savefig( image_filename.replace(".png","_thresholded.png"), dpi=300)
plt.close()

#Wait for user input to close terminal window
input("\n Press return to close ")

#End of code




